@@ -23,6 +23,10 @@
 #	define to_xstring std::to_wstring
 #	define XL(x) L##x
 #	define xcout std::wcout
+#   define to_xlower ::towlower
+#   define to_xupper ::towupper
+#   define is_xupper std::iswupper
+#   define is_xpunct std::iswpunct
 #else
 #	define xstring std::string
 #	define xchar char
@@ -34,4 +38,8 @@
 #	define to_xstring std::to_string
 #	define XL(x) x
 #	define xcout std::cout
+#   define to_xlower ::tolower
+#   define to_xupper ::toupper
+#   define is_xupper std::isupper
+#   define is_xpunct std::ispunct
 #endif
